@@ -458,3 +458,22 @@ pub fn children_names_with_maybe_paren(
         )
     }
 }
+
+pub fn is_definitely_view_on_child(circuit: CircuitRc) -> bool {
+    if circuit.children().count() != 1 {
+        return false;
+    }
+    match &**circuit {
+        Circuit::Rearrange(rr) => {
+            rr.spec.input_ints.iter().all(|x| x.len() < 2)
+                && rr.spec.output_ints.iter().all(|x| x.len() < 2)
+            // this is too conservative
+        }
+        Circuit::Index(idx) => !idx
+            .index
+            .0
+            .iter()
+            .any(|x| matches!(x, TensorAxisIndex::Tensor(_))),
+        _ => false,
+    }
+}
