@@ -121,6 +121,10 @@ pub trait CircuitNode: CircuitNodeInit + CircuitNodeSelfOnlyHash {
     fn info(&self) -> &CachedCircuitInfo;
     fn name(&self) -> Option<&str>;
 
+    /// child -> child dim -> parent dim it's the same as
+    /// same as = index on that dim commutes w/ self
+    /// must be None if not or if unsure
+    /// if any parent dim is Some, then it must be sound to push down index into only the children w/ that dim
     fn child_axis_map(&self) -> Vec<Vec<Option<usize>>>;
 
     fn children(&self) -> Box<dyn Iterator<Item = CircuitRc> + '_>;
@@ -1966,6 +1970,12 @@ impl IntoPy<PyObject> for CircuitRc {
 impl<'source> FromPyObject<'source> for CircuitRc {
     fn extract(circuit_obj: &'source PyAny) -> PyResult<Self> {
         {
+            if !circuit_obj.is_instance_of::<PyCircuitBase>()? {
+                return Err(PyErr::new::<pyo3::exceptions::PyTypeError, _>(format!(
+                    "object of {} cannot be converted to 'Circuit'",
+                    circuit_obj.get_type()
+                )));
+            }
             let circ: Circuit = circuit_obj.extract()?;
             Ok(circ.rc())
         }
