@@ -1,4 +1,9 @@
-use std::{collections::BTreeSet, fmt::Debug, ops::Deref, sync::Arc};
+use std::{
+    collections::{BTreeMap, BTreeSet},
+    fmt::Debug,
+    ops::Deref,
+    sync::Arc,
+};
 
 use anyhow::Result;
 use pyo3::{prelude::*, pyclass::CompareOp, types::PyDict, PyObject};
@@ -8,7 +13,7 @@ use rr_util::{
     pycall,
 };
 
-use crate::CircuitRc;
+use crate::{CircuitRc, CircuitType};
 
 #[derive(Debug, Clone)]
 pub enum OpaqueIterativeMatcherVal {
@@ -346,6 +351,34 @@ impl OpaqueIterativeMatcherVal {
 
         fn get_unique(&self, circuit: CircuitRc, fancy_validate: bool) -> PyResult<CircuitRc>;
 
+        fn get_paths(&self, circuit: CircuitRc) -> PyResult<BTreeMap<CircuitRc, Vec<CircuitRc>>>;
+        fn get_all_paths(
+            &self,
+            circuit: CircuitRc,
+        ) -> PyResult<BTreeMap<CircuitRc, Vec<Vec<CircuitRc>>>>;
+
         fn are_any_found(&self, circuit: CircuitRc) -> PyResult<bool>;
     );
 }
+
+pub fn get_opaque_type_matcher(
+    circuit_type: CircuitType,
+    traversal: Option<OpaqueIterativeMatcherVal>,
+) -> OpaqueIterativeMatcherVal {
+    Python::with_gil(|py| {
+        let kwargs = PyDict::new(py);
+        let not_type_matcher = SELF_MODULE
+            .getattr(py, "Matcher")
+            .unwrap()
+            .call1(py, (circuit_type,))
+            .unwrap()
+            .call_method0(py, "new_not")
+            .unwrap();
+        kwargs.set_item("term_early_at", not_type_matcher).unwrap();
+        OpaqueIterativeMatcherVal::op_to_object(&traversal)
+            .call_method(py, "filter", (), Some(kwargs))
+            .unwrap()
+            .extract(py)
+            .unwrap()
+    })
+}
