@@ -14,7 +14,7 @@ use rr_util::{
     py_types::{is_python_running, Tensor, PY_UTILS},
     python_println,
     repr::RustRepr,
-    tensor_util::Shape,
+    tensor_util::{Shape, TorchDeviceDtype},
     timed,
     util::{apply_fn_until_same, HashBytes},
 };
@@ -75,6 +75,10 @@ pub struct OptimizationSettings {
     pub log_slow_einsums: bool,
     #[pyo3(get, set)]
     pub save_circ: bool,
+    #[pyo3(get, set)]
+    pub optimization_parallelism: Option<usize>,
+    #[pyo3(get, set)]
+    pub device_dtype: Option<TorchDeviceDtype>,
 }
 
 impl Default for OptimizationSettings {
@@ -97,6 +101,8 @@ impl Default for OptimizationSettings {
             log_simplifications: false,
             log_slow_einsums: false,
             save_circ: false,
+            optimization_parallelism: Some(8),
+            device_dtype: None,
         }
     }
 }
@@ -121,7 +127,9 @@ impl OptimizationSettings {
         create_tests = "OptimizationSettings::default().create_tests",
         log_simplifications = "OptimizationSettings::default().log_simplifications",
         log_slow_einsums = "OptimizationSettings::default().log_slow_einsums",
-        save_circ = "OptimizationSettings::default().save_circ"
+        save_circ = "OptimizationSettings::default().save_circ",
+        optimization_parallelism = "None",
+        device_dtype = "None"
     )]
     fn new(
         verbose: usize,
@@ -141,6 +149,8 @@ impl OptimizationSettings {
         log_simplifications: bool,
         log_slow_einsums: bool,
         save_circ: bool,
+        optimization_parallelism: Option<usize>,
+        device_dtype: Option<TorchDeviceDtype>,
     ) -> Self {
         Self {
             verbose,
@@ -160,6 +170,8 @@ impl OptimizationSettings {
             log_simplifications,
             log_slow_einsums,
             save_circ,
+            optimization_parallelism,
+            device_dtype,
         }
     }
 }
@@ -333,7 +345,7 @@ pub fn optimize_to_schedule(
     let optimized_circuit = timed!(optimize_circuit(circuit, context), 10, verbose >= 1)?;
     // return evaluate_fn(&*optimized_circuit).unwrap();
     let schedule = if settings.scheduling_naive {
-        circuit_to_schedule_naive_toposort(optimized_circuit)
+        circuit_to_schedule_naive_toposort(optimized_circuit, context.settings.device_dtype)?
     } else {
         timed!(
             circuit_to_schedule(optimized_circuit, context),
