@@ -144,6 +144,26 @@ pub fn module_remove_unused_inputs(
     )
 }
 
+// not a rewrite, doesn't keep shape!
+pub fn module_strip_args(m: &Module) -> Module {
+    let used_inputs = m.spec.are_args_used();
+    let (nodes, arg_specs) = m
+        .arg_items()
+        .into_iter()
+        .zip(used_inputs)
+        .filter_map(|(x, used)| used.then_some(x))
+        .unzip();
+
+    Module::new(
+        nodes,
+        ModuleSpec {
+            circuit: m.spec.circuit.clone(),
+            arg_specs,
+        },
+        m.name_cloned(),
+    )
+}
+
 #[pyfunction(
     prefix_to_strip = "None",
     module_name = "None",
