@@ -1,8 +1,9 @@
-use std::{hash::Hash, iter::zip};
+use std::iter::zip;
 
 use circuit_base::{prelude::*, Add, Concat, Einsum, Index, Rearrange, Scalar, Scatter};
 use pyo3::prelude::*;
 use rr_util::{
+    compact_data::TinyVecU8,
     rearrange_spec::RearrangeSpec,
     tensor_util::{
         compose, uslices_shrink_base, uslices_to_index, Shape, TensorAxisIndex, TensorIndex, USlice,
@@ -10,7 +11,6 @@ use rr_util::{
     util::{cumsum, filter_out_idx},
 };
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
-use smallvec::{Array, SmallVec as Sv};
 
 use crate::algebraic_rewrite::{get_removable_axes, remove_axes};
 
@@ -31,13 +31,7 @@ pub fn scatter_fuse(scatter: &Scatter) -> Option<Scatter> {
     }
 }
 
-pub fn uslice_map_get_index<A: Array>(
-    ints: &Sv<A>,
-    int_slices: &HashMap<A::Item, USlice>,
-) -> TensorIndex
-where
-    A::Item: Eq + Hash,
-{
+pub fn uslice_map_get_index(ints: &TinyVecU8, int_slices: &HashMap<u8, USlice>) -> TensorIndex {
     TensorIndex(
         ints.iter()
             .map(|i| {
