@@ -7,7 +7,7 @@ use rr_util::{
     lru_cache::TensorCacheRrfs,
     py_types::{un_flat_concat, Tensor, PY_UTILS},
     pycall, sv,
-    tensor_util::{Shape, TorchDeviceDtype},
+    tensor_util::{Shape, TorchDevice, TorchDeviceDtype, TorchDtype},
 };
 use rustc_hash::FxHashMap as HashMap;
 
@@ -36,7 +36,7 @@ pub struct ScheduleToSend {
 
 #[pymethods]
 impl ScheduleToSend {
-    pub fn evaluate_remote(&self, remote_url: String, device: String) -> Option<Tensor> {
+    pub fn evaluate_remote(&self, remote_url: String, device: TorchDevice) -> Option<Tensor> {
         let self_string = json::to_string(&self);
         let response = ureq::post(&remote_url).send_string(&self_string).unwrap();
         let mut body: Vec<u8> = vec![];
@@ -50,7 +50,7 @@ impl ScheduleToSend {
             PY_UTILS.tensor_from_bytes,
             (
                 TorchDeviceDtype {
-                    dtype: self.dtype.clone(),
+                    dtype: TorchDtype::try_from(self.dtype.clone()).unwrap(),
                     device
                 },
                 out_shape,
@@ -59,7 +59,11 @@ impl ScheduleToSend {
             )
         )
     }
-    pub fn evaluate_remote_many(&self, remote_url: String, device: String) -> Option<Vec<Tensor>> {
+    pub fn evaluate_remote_many(
+        &self,
+        remote_url: String,
+        device: TorchDevice,
+    ) -> Option<Vec<Tensor>> {
         self.evaluate_remote(remote_url, device).map(|tensor| {
             un_flat_concat(
                 &tensor,
@@ -76,14 +80,18 @@ impl ScheduleToSend {
 }
 
 impl ScheduleToSend {
-    pub fn load(self, device: String, cache: &mut Option<TensorCacheRrfs>) -> Result<Schedule> {
+    pub fn load(
+        self,
+        device: TorchDevice,
+        cache: &mut Option<TensorCacheRrfs>,
+    ) -> Result<Schedule> {
         let mut result = Schedule {
             instructions: vec![],
             constants: Default::default(),
             scalars: Default::default(),
             device_dtype: TorchDeviceDtype {
                 device,
-                dtype: self.dtype,
+                dtype: TorchDtype::try_from(self.dtype.clone()).unwrap(),
             },
             output_circuit: Some((self.output_circuit.0, Scalar::nrc(0.0, sv![], None))),
             split_shapes: self
@@ -198,7 +206,7 @@ impl TryFrom<&Schedule> for ScheduleToSend {
                 .collect::<Result<_>>()?,
             constants: map_to_str(&x.constants)?,
             scalars: map_to_str(&x.scalars)?,
-            dtype: x.device_dtype.dtype.clone(),
+            dtype: String::from(&x.device_dtype.dtype),
             output_circuit: (
                 x.output_circuit.clone().unwrap().0,
                 x.output_circuit
