@@ -344,10 +344,17 @@ pub enum SchedulingOOMError {
     },
     #[error("Single element doesn't fit {formatted} ({e_name})")]
     Single { numel: BigUint, formatted: String },
-    #[error(
-        "Simplification can't find anything within bounds, at {origin} graph {cyto} ({e_name})"
-    )]
-    Simp { cyto: String, origin: String },
+    #[error("Simplification can't find anything within bounds, likely each isolated subdag fits but together they don't. maybe try scheduling_simplify=False opt setting graph {cyto} ({e_name})")]
+    Simp { cyto: String },
+    #[error("Threading failed! bug! ({e_name})")]
+    ThreadsLost {},
+
+    #[error("Schedule exhaustive timed out, at size {size} iters {iters} graph (click to see circuit structure) {cyto} ({e_name})")]
+    ExhaustiveTimeout {
+        cyto: String,
+        iters: usize,
+        size: usize,
+    },
 }
 
 #[apply(python_error_exception)]
@@ -812,6 +819,7 @@ pub fn circuit_to_schedule(
                 dag_simp_settings.num_mem_chunks = context.settings.scheduling_num_mem_chunks;
                 dag_simp_settings.timeout = context.settings.scheduling_timeout;
                 dag_simp_settings.verbose = context.settings.verbose;
+                dag_simp_settings.parallelism = context.settings.optimization_parallelism;
                 timed!(
                     dag.simplify(&dag_simp_settings)?,
                     10,
@@ -840,6 +848,13 @@ pub fn circuit_to_schedule(
         .iter()
         .map(|x| to_node[&dag.node_hashes[*x as usize]].clone())
         .collect();
+    if let Some(passed_dd) = context.settings.device_dtype {
+        circuit
+            .info()
+            .device_dtype
+            .combine(passed_dd.into())
+            .context("circuit device_dtype must match OptimizationSettings device_dtype")?;
+    }
     let (mut out, kept_keys) = order_to_schedule(
         &circuit_order,
         &to_node
@@ -851,7 +866,10 @@ pub fn circuit_to_schedule(
             .filter_map(|x| x.1.as_scalar().cloned())
             .collect(),
         HashSet::from_iter([circuit.info().hash]),
-        get_compatible_dtype(&circuit),
+        context
+            .settings
+            .device_dtype
+            .unwrap_or_else(|| get_compatible_dtype(&circuit)),
     );
     out.output_circuit = Some((kept_keys[0], circuit));
     let out = replace_module_schedules(&out, context);
@@ -859,7 +877,10 @@ pub fn circuit_to_schedule(
     Ok(out)
 }
 
-pub fn circuit_to_schedule_naive_toposort(circuit: CircuitRc) -> Schedule {
+pub fn circuit_to_schedule_naive_toposort(
+    circuit: CircuitRc,
+    device_dtype: Option<TorchDeviceDtype>,
+) -> Result<Schedule> {
     let circuit = substitute_all_modules(circuit);
     let toposorted = toposort_circuit(circuit.clone());
     let mut order = Vec::new();
@@ -875,23 +896,29 @@ pub fn circuit_to_schedule_naive_toposort(circuit: CircuitRc) -> Schedule {
             order.push(c)
         }
     }
-
+    if let Some(device_dtype) = device_dtype {
+        circuit
+            .info()
+            .device_dtype
+            .combine(device_dtype.into())
+            .context("circuit device_dtype must match OptimizationSettings device_dtype")?;
+    }
     let (mut result, kept_keys) = order_to_schedule(
         &order,
         &constants,
         &scalars,
         HashSet::from_iter([circuit.info().hash]),
-        get_compatible_dtype(&circuit),
+        device_dtype.unwrap_or_else(|| get_compatible_dtype(&circuit)),
     );
     result.output_circuit = Some((kept_keys[0], circuit));
     result.validate(true).expect("should be valid");
-    result
+    Ok(result)
 }
 
 #[pyfunction]
 pub fn scheduled_evaluate(circuit: CircuitRc, settings: OptimizationSettings) -> Result<Tensor> {
     let schedule = if settings.scheduling_naive {
-        Ok(circuit_to_schedule_naive_toposort(circuit))
+        circuit_to_schedule_naive_toposort(circuit, settings.device_dtype)
     } else {
         circuit_to_schedule(
             circuit.clone(),
