@@ -116,7 +116,7 @@ impl RegexWrap {
     #[args(escape_dot = "true")]
     pub fn new(pattern: String, escape_dot: bool) -> Result<Self> {
         let new_pattern = if escape_dot {
-            Regex::new(r"(\.)|(\\\.)")
+            Regex::new(r"(\\\.)|(\.)")
                 .unwrap()
                 .replace_all(&pattern, |captures: &Captures| {
                     if captures.get(1).is_some() {
