@@ -12,7 +12,9 @@ use circuit_base::{
 };
 use circuit_rewrites::{
     deep_rewrite::SimpFnSubset,
-    module_rewrite::{elim_no_input_module, extract_rewrite_raw, module_remove_unused_inputs},
+    module_rewrite::{
+        elim_no_input_module, extract_rewrite_raw, module_remove_unused_inputs, module_strip_args,
+    },
 };
 use get_update_node::{AnyFound, IterativeMatcher, IterativeMatcherRc, MatcherData};
 use itertools::multiunzip;
@@ -1132,6 +1134,8 @@ impl ModuleConstructUpdateBindings {
             return Ok(m.rc());
         }
 
+        let m = self.deep_remove_unused(m, overall_count);
+
         self.bind_module(
             m,
             overall_count,
@@ -1142,6 +1146,27 @@ impl ModuleConstructUpdateBindings {
         .map(|(c, _, _)| c)
     }
 
+    fn deep_remove_unused(&self, m: Module, current_count: usize) -> Module {
+        let m = if current_count > 1 && !self.is_flatten {
+            let new_spec_circuit = self
+                .deep_remove_unused(m.spec.circuit.as_module_unwrap().clone(), current_count - 1)
+                .rc();
+            let name = m.name_cloned();
+            Module::new(
+                m.nodes,
+                ModuleSpec {
+                    circuit: new_spec_circuit,
+                    arg_specs: m.spec.arg_specs,
+                },
+                name,
+            )
+        } else {
+            m
+        };
+
+        module_strip_args(&m)
+    }
+
     fn bind_module(
         &mut self,
         m: Module,
@@ -1171,6 +1196,7 @@ impl ModuleConstructUpdateBindings {
             m
         };
 
+        // this assumes that unused args of children were already removed
         let used_inputs = m.spec.are_args_used();
         let (nodes, arg_specs): (Vec<_>, Vec<_>) = m
             .arg_items()
