@@ -1,4 +1,7 @@
-use std::{num::NonZeroUsize, ops::Deref};
+use std::{
+    num::NonZeroUsize,
+    ops::{Deref, DerefMut},
+};
 
 use pyo3::{FromPyObject, IntoPy, PyAny, PyObject, PyResult, Python};
 
@@ -125,6 +128,11 @@ impl Deref for TinyVecU8 {
         self.as_slice()
     }
 }
+impl DerefMut for TinyVecU8 {
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        self.as_mut_slice()
+    }
+}
 
 impl Drop for TinyVecU8 {
     fn drop(&mut self) {
