@@ -1,5 +1,6 @@
 use std::{
     cmp::{max, min},
+    iter::zip,
     rc::Rc,
 };
 
@@ -26,8 +27,9 @@ use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 use crate::{
     tensor_util::ParseError,
     union_find::UnionFind,
-    util::{dict_to_list, EinsumAxes, ALPHABET, ALPHABET_INV},
+    util::{dict_to_list, EinsumAxes, NamedAxes, ALPHABET, ALPHABET_INV},
 };
+
 /// returns vec of vecs of indices of operands
 pub fn get_disconnected_sugraphs(
     operand_ints: &Vec<Vec<usize>>,
@@ -293,6 +295,74 @@ impl EinsumSpec {
             .join("");
         operand_letters + "->" + &out_letters
     }
+
+    /// Use self and children's named axes for axes names. Return None if not enough named axes.
+    /// If different named are given to the same axes, use the axes given by self (highest priority), or the first child, or the second, ...
+    pub fn to_fancy_einsum_string(
+        &self,
+        self_named_axes: NamedAxes,
+        children_named_axes: Vec<NamedAxes>,
+    ) -> Option<String> {
+        let mut axis_to_name: HashMap<usize, &String> = Default::default();
+
+        // self named axes have priority
+        for (i, name) in self_named_axes.iter() {
+            axis_to_name.insert(self.output_ints[*i as usize], name);
+        }
+
+        let operand_words = zip(children_named_axes.iter(), self.input_ints.iter())
+            .map(|(named_axes, one_operand_ints)| {
+                if named_axes.len() < one_operand_ints.len() {
+                    return None; // No hope of having a name for each axis
+                }
+                Some(
+                    one_operand_ints
+                        .iter()
+                        .enumerate()
+                        .map(|(i, a)| {
+                            if let Some(o_name) = axis_to_name.get(a) {
+                                Some((*o_name).clone())
+                            } else {
+                                let name = named_axes.get(&(i as u8))?;
+                                axis_to_name.insert(*a, name);
+                                Some(name.clone())
+                            }
+                        })
+                        .collect::<Option<Vec<String>>>()?
+                        .join(" "),
+                )
+            })
+            .collect::<Option<Vec<String>>>()?
+            .join(", ");
+        if axis_to_name.len() > HashSet::from_iter(axis_to_name.values().cloned()).len() {
+            return None; // Can't tolerate duplicate names
+        }
+        let out_words = self
+            .output_ints
+            .iter()
+            .map(|a| (*axis_to_name.get(a).unwrap()).clone()) // output ints included in input ints
+            .collect::<Vec<String>>()
+            .join(" ");
+        Some(format!("fancy: {operand_words} -> {out_words}"))
+    }
+
+    pub fn to_maybe_fancy_string(
+        &self,
+        self_named_axes: NamedAxes,
+        children_named_axes: Vec<NamedAxes>,
+    ) -> String {
+        if self_named_axes.is_empty() {
+            self.to_einsum_string() // in this case we override and don't use fancy which
+                                    // just adds extra stuff we don't want
+        } else if let Some(fancy_string) =
+            self.to_fancy_einsum_string(self_named_axes, children_named_axes)
+        {
+            fancy_string
+        } else {
+            self.to_einsum_string()
+        }
+    }
+
     pub fn flops(&self) -> BigUint {
         if self.input_ints.is_empty() {
             return BigUint::from(0usize);
