@@ -133,6 +133,9 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         "traversal",
         get_update_node::IterativeMatcher::noop_traversal(),
     )?;
+    m.add_function(wrap_pyfunction!(get_update_node::restrict, m)?)?;
+    m.add_function(wrap_pyfunction!(get_update_node::restrict_sl, m)?)?;
+    m.add_function(wrap_pyfunction!(get_update_node::new_traversal, m)?)?;
     m.add_class::<get_update_node::IterateMatchResults>()?;
     m.add_class::<circuit_base::opaque_iterative_matcher::Finished>()?;
     m.add("FINISHED", circuit_base::opaque_iterative_matcher::Finished)?;
@@ -511,6 +514,14 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         nb_operations::modules::default_nested_module_namer,
         m
     )?)?;
+    m.add_function(wrap_pyfunction!(
+        nb_operations::modules::default_update_bindings_nested_namer,
+        m
+    )?)?;
+    m.add_function(wrap_pyfunction!(
+        nb_operations::modules::update_bindings_nested,
+        m
+    )?)?;
     m.add_function(wrap_pyfunction!(
         nb_operations::modules::extract_symbols,
         m
@@ -551,6 +562,8 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         "CircuitColorer",
         "CircuitHtmlColorer",
         "NestedModuleNamer",
+        "ModuleConstructCallback",
+        "MaybeUpdate",
     ] {
         m.add(dummy_name, py.None())?;
     }
